# encoding: utf-8

"""
CLIRch config module: main classes
"""

from inspect import ismodule
import logging

from clirch.core import logger
from clirch.language.registry import LanguageRegistry

class Config(object):
    """
    The main config object.

    To be instanced and passed to CLIRch main constructor.

    Can take options in two ways: Dictionary or settings module/object.

    As a default with no options given, will load all languages defined in
    clirch.languages module
    """
    def __init__(self, override_dict=None, settings_from=None):
        self.settings_from = settings_from
        settings = {
                'languages': None,
                'pipeline_overrides': dict(),
                'loglevel': logging.DEBUG, # TODO: Should default to logging.INFO at some time
                'logfile': None,
                }

        if not settings_from is None:
            if isinstance(settings_from, basestring):
                try:
                    settings_from = __import__(settings_from, fromlist='*')
                except ImportError:
                    raise ImportError('''Could not load the "%s" module.

We tried to load instance settings for CLIRch using this module. It
appears to have failed. Either supply options using a dictionary, or make sure
you are trying to load settings from a module that exists.''' % settings_from)

            if not isinstance(settings_from, dict):
                settings_from = dict((key, value) for key, value in
                        settings_from.__dict__.iteritems() if not
                        ismodule(value))

            settings.update(settings_from)

        if not override_dict is None:
            settings.update(override_dict)

        for key, value in settings.iteritems():
            setattr(self, key, value)

        if self.loglevel < logging.INFO:
            log_fmt = '[%(levelname)8s] %(name)12s %(asctime)s: %(message)s'
        else:
            log_fmt = '%(levelname)8s: %(message)s'

        if self.logfile:
            logging.basicConfig(
                    level=self.loglevel,
                    format=log_fmt,
                    filename=self.logfile,
                    )
        else:
            logging.basicConfig(
                    level=self.loglevel,
                    format=log_fmt,
                    )

        if not isinstance(self.languages, LanguageRegistry):
            if self.languages:
                self.languages = LanguageRegistry(self.languages, self)
            else:
                self.languages = LanguageRegistry(config=self)

        logger.debug('Set up the logger to level: %s' %
                logging.getLevelName(logging.getLogger().level))
