# encoding: utf-8

"""
Holds Pipeline classes.
"""

import logging

from clirch.core import logger

class Pipeline(object):
    """
    The Pipeline class. Holds a list of callable objects to be used for
    translating a specific language pair.

    The object itself is callable, and normally expects a string.
    """
    def __init__(self, callables):
        self.callables = callables

    def __call__(self, words):

        for partial in self.callables:
            logger.debug('Calling %r with words %r' % (partial, words))
            words = partial(words)
            if logger.getEffectiveLevel() <= logging.DEBUG and hasattr(words, 'next'):
                words = list(words)
        return words

class Pipelines(object):
    """
    The Pipelines decides the Pipeline that is used for search phrases. It is
    specific for an input language, and may hold Pipeline objects for for
    several output languages.

    If the Pipelines object holds a Pipeline for "Spanish" and "Norwegian", you
    can find the Pipeline as pipelines_instance.to_languages['no'] or as
    pipelines_instance.to_norwegian.
    """

    def register_language(self, target_lang, pipeline):
        self.to_languages[target_lang.code] = pipeline
        setattr(self, 'to_%s' % target_lang.name.lower(), pipeline)

    def __init__(self, language, config_instance):
        self.active_langs = config_instance.languages
        self.pipeline_overrides = config_instance.pipeline_overrides

        self.language = language
        self.to_languages = dict()

        if language in config_instance.pipeline_overrides:
            for target_lang, override in \
                    self.pipeline_overrides[language].iteritems():
                self.register_language(target_lang, Pipeline(override))

        else:
            for target_lang in self.active_langs.values():
                pipe = language.pipes.get(target_lang, None)
                if pipe is None:
                    raise KeyError('No pipeline defined for %s to %s' %
                            (language, target_lang))

                self.register_language(target_lang, Pipeline(pipe))
