# encoding: utf-8

from __future__ import with_statement

from collections import defaultdict
import os
import re
import shutil
import time
from urllib import urlencode
import urllib2
try:
    import cPickle as pickle
except ImportError:
    import pickle

from nltk.containers import Trie

from clirch.core import logger
from clirch.utils.word import TransformedWord
import dictionary

PICKLE_FILE = os.path.join(os.path.dirname(__file__), 'tritrans.tries.pickle')

_cache_loaded = False
_cache = {
        'no': defaultdict(Trie),
        'en': defaultdict(Trie),
        'es': defaultdict(Trie)
    }

def load_cache():
    if os.path.exists(PICKLE_FILE):
        logger.debug('Loading tritrans cache from ' + PICKLE_FILE)
        with open(PICKLE_FILE, 'r') as fh:
            global _cache
            _cache = pickle.load(fh)
    else:
        logger.debug('File not found: ' + PICKLE_FILE)

def save_cache():
    if os.path.exists(PICKLE_FILE):
        shutil.move(PICKLE_FILE, PICKLE_FILE + '_')
    with open(PICKLE_FILE, 'w') as fh:
        pickle.dump(_cache, fh)

def tritrans(lang_code, lookup, save=True):
    logger.debug('Looking up "%s" from language "%s" on tritrans.net' % (lookup, lang_code))
    URL = 'http://www.tritrans.net/cgibin/translate.cgi?'
    pattern = r'<tr><td><li>(?P<lang>.*):</li></td><td><font color="#000000">(?P<words>.*)</font></td></tr>'
    lang = {
            'no': 'Norsk',
            'es': 'Spansk',
            'en': 'Engelsk'
        }[lang_code]

    lookup_enc = lookup
    if isinstance(lookup, unicode):
        try:
            lookup_enc = lookup.encode('iso-8859-1').decode('utf-8').encode('iso-8859-1')
        except UnicodeDecodeError:
            lookup_enc = lookup.encode('iso-8859-1')

    params = urlencode([
        ('spraak', lang),
        ('Fra', lookup_enc),
        ('button', 'Translate!')
        ])
    request = urllib2.Request(URL + params)
    request.add_header('Host', 'www.tritrans.net')
    request.add_header('User-Agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:7.0.1) Gecko/20100101 Firefox/7.0.1')
    request.add_header('Accept-Language', 'en-gb,en;q=0.8,nb;q=0.6,no;q=0.4,nn;q=0.2')
    request.add_header('Accept-Encodinggzip', 'deflate')
    request.add_header('Accept-Charset', 'ISO-8859-1,utf-8;q=0.7,*;q=0.7')
    request.add_header('Connection', 'keep-alive')
    request.add_header('Referer', 'http://www.tritrans.net/')

    tries = 0
    while tries < 30:
        try:
            response = urllib2.urlopen(request)
        except urllib2.HTTPError, ex:
            logger.warn('urllib2.HTTPError: ' + str(ex))
            time.sleep(3)
        except urllib2.URLError:
            logger.warning('urllib2.URLError: ' + str(ex))
            time.sleep(15)
        tries += 1

    for line in response:
        match = re.match(pattern, line)
        if match:
            match_lang, words = match.groups()
            # Ugly latin1 is ugly
            words = unicode(words, 'iso-8859-1')
            match_lang = {
                    'English': 'en',
                    'Spanish': 'es',
                    'Norwegian': 'no'
                }[match_lang]
            if 'http://' in words:
                found = re.match(r'.*">([^<]+)</a>.*', words)
                if found:
                    found = found.groups()[0]
                    if not found in _cache[lang_code][match_lang]:
                        tritrans(lang_code, found, False)
                    if _cache[lang_code][match_lang].isleaf(found):
                        _cache[lang_code][match_lang][lookup] = \
                                _cache[lang_code][match_lang][found]
                    logger.debug('Using "%s" as alias for "%s" in "%s"' % (found, lookup, match_lang))
                continue
            logger.debug('Saving "%s" in "%s"' % (words, match_lang))
            result = [word.strip() for word in words.split(',')]
            _cache[lang_code][match_lang][lookup] = filter(lambda word: word and not word.startswith('('), result)
    if save:
        save_cache()

class Tritrans(dictionary.Dictionary):
    """
    Ugly hack for looking up words on http://www.tritrans.net/
    """
    def __init__(self, from_lang, to_lang, auto_save=True, look_up=False, passthrough=True):
        global _cache_loaded
        if not _cache_loaded:
            _cache_loaded = True
            logger.debug('Loading tritrans cache automatically')
            load_cache()
            logger.debug('Loading tritrans cache: DONE')
        self.from_lang = from_lang
        self.to_lang = to_lang
        self.auto_save = auto_save
        self.look_up = look_up
        self.passthrough = passthrough
        self._cache = _cache[from_lang][to_lang]

    def __call__(self, words):
        for word in words:
            if self.look_up and not self._cache.isleaf(word):
                tritrans(self.from_lang, word, self.auto_save)

            if self._cache.isleaf(word):
                result = self._cache[word]
            elif self.passthrough:
                result = [word]
            else:
                result = []

            logger.debug('Tritrans: %s -> %s' % (repr(word), repr(result)))
            for translation in result:
                yield TransformedWord(translation, word)

class NonPassthroughTritrans(Tritrans):
    def __init__(self, *args, **kwargs):
        super(NonPassthroughTritrans, self).__init__(*args, passthrough=False, **kwargs)
